package si.uni_lj.fe.tnuv.pametnabudilka;

import android.graphics.Color;
import android.os.Bundle;

import androidx.appcompat.app.AppCompatActivity;

import com.github.mikephil.charting.charts.BarChart;
import com.github.mikephil.charting.components.Legend;
import com.github.mikephil.charting.components.XAxis;
import com.github.mikephil.charting.components.YAxis;
import com.github.mikephil.charting.data.BarData;
import com.github.mikephil.charting.data.BarDataSet;
import com.github.mikephil.charting.data.BarEntry;
import com.github.mikephil.charting.formatter.IndexAxisValueFormatter;
import com.github.mikephil.charting.formatter.ValueFormatter;
import com.github.mikephil.charting.utils.ColorTemplate;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;

public class BarGraph extends AppCompatActivity {
    ArrayList<String[]> rawData = new ArrayList<String[]>();
    ArrayList<String[]> data = new ArrayList<String[]>();


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_bar_graph);

        BarChart barChart = findViewById(R.id.barChart);

        String sleepStart = getIntent().getStringExtra("sleepStart");
        String sleepStop = getIntent().getStringExtra("sleepStop");

        ReadFile file = new ReadFile(getApplicationContext(), sleepStart +".txt");
        try {
            rawData = file.getData();
        } catch (IOException e) {
            e.printStackTrace();
        }

        //uredi disruptorje
        for(int i = 0; i<rawData.size();i++){
            if ( !(rawData.get(i)[1].equalsIgnoreCase("idle")) ) {
                data.add(rawData.get(i));
            }
        }


        //dobimo ven podatke za uro zacetka spanja
        Calendar startTime = Calendar.getInstance();
        startTime.setTimeInMillis(Long.parseLong(sleepStart));
        long startHour = startTime.get(Calendar.HOUR_OF_DAY);
        //Log.i("ura_zacetka", String.valueOf(startHour));


        //izracun dolzine spanja
        int sleepHours = (int) ((Long.parseLong(sleepStop) - Long.parseLong(sleepStart))/3600000 + 2);
        //Log.i("dolzina spanja", String.valueOf(sleepHours));

        ArrayList<BarEntry> disruptors = new ArrayList<>();

        //stetje motilcev po urah
        ArrayList hoursList = new ArrayList();
        int[] disruptorsPerHour = new int[sleepHours];
        for (int i=0;i<sleepHours;i++){
            for (int j=0;j<data.size();j++){
                Calendar calendar = Calendar.getInstance();
                calendar.setTimeInMillis(Long.parseLong(data.get(j)[0]));

                if((startHour + i) % 24 == calendar.get(Calendar.HOUR_OF_DAY)){
                    disruptorsPerHour[i] += 1;
                }
            }
            //Log.i("disruptorji", (startHour + i) % 24 + "  " + disruptorsPerHour[i]);
            hoursList.add(String.valueOf((startHour + i) % 24) + "h");
            disruptors.add(new BarEntry(i +1,disruptorsPerHour[i]));

        }


        BarDataSet barDataSet = new BarDataSet(disruptors,"disruptors each hour");
        barDataSet.setColors(ColorTemplate.MATERIAL_COLORS);
        barDataSet.setValueTextColor(Color.WHITE);
        barDataSet.setValueTextSize(18f);

        //Value formatter za izpis integerjev namesto floatov
        ValueFormatter valueFormatter = new ValueFormatter() {
            @Override
            public String getFormattedValue(float value) {
                return ""+(int)value;
            }
        };


        BarData barData = new BarData(barDataSet);

        barData.setValueFormatter(valueFormatter);
        barChart.setData(barData);
        barChart.getDescription().setText("");
        barChart.animateY(2000);
        barChart.setExtraBottomOffset(10f);

        //legenda
        Legend legend = barChart.getLegend();
        legend.setTextSize(20f);
        legend.setTextColor(Color.WHITE);
        legend.setVerticalAlignment(Legend.LegendVerticalAlignment.TOP);

        //x-axis customization
        XAxis xAxis = barChart.getXAxis();
        xAxis.setValueFormatter(new IndexAxisValueFormatter(hoursList));
        xAxis.setPosition(XAxis.XAxisPosition.BOTTOM);
        xAxis.setTextSize(15f);
        xAxis.setTextColor(Color.WHITE);
        xAxis.setCenterAxisLabels(true);
        xAxis.setLabelCount(9);

        //y-axis customization
        YAxis yAxisLeft = barChart.getAxisLeft();
        yAxisLeft.setEnabled(false);
        yAxisLeft.setAxisMinimum(0f);

        YAxis yAxisRight = barChart.getAxisRight();
        yAxisRight.setEnabled(false);

        //izklopimo grid
        barChart.getAxisLeft().setDrawGridLines(false);
        barChart.getAxisRight().setDrawGridLines(false);
        barChart.getXAxis().setDrawGridLines(false);


    }
}