package si.uni_lj.fe.tnuv.pametnabudilka;

import android.Manifest;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattDescriptor;
import android.bluetooth.BluetoothGattService;
import android.bluetooth.BluetoothManager;
import android.bluetooth.BluetoothProfile;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.RequiresApi;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import static android.bluetooth.BluetoothGattCharacteristic.FORMAT_SINT16;

public class SleepingActivity extends AppCompatActivity {

    private static final String DEVICE_NAME = "Pametna budilka V1";
    private static final String SERVICE_UUID = "30dab7a0-97d7-4f83-8efa-a3e2c237ac88";
    private static final String CHAR_UUID = "fad9b842-8945-4222-8876-0426065469db";
    private static final int REQUEST_ENABLE_BT = 1;
    private BluetoothAdapter bluetoothAdapter;
    private BluetoothGatt gatt;

    //timer
    int seconds=0;

    TextView printDisruptorText;
    File file;

    //buffer za pisat, da ne odpira/zapira skos fajla
    String[] textWriteBuffer = new String[50];
    int posLimit = 10;
    int checkBufferInterval = 10;
    int currentPos = 0;

    //tabela za pretvorbo iz vrednosti prejete preko BT v motilec
    Map<Integer, String> intToStringTable = new HashMap<>();
    //Map<Integer, String> intToStringTable = new HashMap<Integer, String>();
    int BTrecieved;
    String disruptor;

    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_sleeping);
        //Log.i("Test", "123");

        ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.ACCESS_COARSE_LOCATION}, 1001);

        intToStringTable.put(-1, "unknown");
        intToStringTable.put(0, "idle");
        intToStringTable.put(1, "idle");
        intToStringTable.put(2, "barking");
        intToStringTable.put(3, "switch");
        intToStringTable.put(4, "doors");
        intToStringTable.put(5, "barking");
        intToStringTable.put(6, "car");
        intToStringTable.put(7, "windows");
        intToStringTable.put(8, "switch");
        intToStringTable.put(9, "snoring");
        intToStringTable.put(10, "switch");

        printDisruptorText = findViewById(R.id.displayDisruptorTV);

        //naredi nov file
        Calendar timeStart = Calendar.getInstance();
        file = new File(this.getFilesDir(), timeStart.getTimeInMillis() + ".txt");
        try {
            file.createNewFile();
        } catch (IOException e) {
            e.printStackTrace();
        }

        final BluetoothManager mBluetoothManager = (BluetoothManager) getSystemService(Context.BLUETOOTH_SERVICE);
        bluetoothAdapter = mBluetoothManager.getAdapter();
        if (bluetoothAdapter == null) {
            Toast.makeText(getApplicationContext(), "Ble not supported", Toast.LENGTH_SHORT).show();
            finish();
            return;
        }

        if (!bluetoothAdapter.isEnabled()) {
            Intent enableBtIntent = new Intent(
                    BluetoothAdapter.ACTION_REQUEST_ENABLE);
            startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT);
        }

        scanLeDevice();

        //pogleda ce buffer napolnjen do meje, da prepise v file
        ScheduledExecutorService executor = Executors.newSingleThreadScheduledExecutor();
        Runnable checkForWrite = new Runnable() {
            @Override
            public void run() {
                //Log.i("Run:", "triggered");

                //pogleda ce naj sprazni buffer
                if (currentPos >= posLimit) {
                    try {
                        //Log.i("currentPos >= posLimit", "writing data...");
                        fileWrite();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }


                seconds++;
            }
        };
        executor.scheduleAtFixedRate(checkForWrite, 0, checkBufferInterval, TimeUnit.SECONDS);

        //izpis preteklega casa na zaslonu
        ScheduledExecutorService executorTimer = Executors.newSingleThreadScheduledExecutor();
        Runnable setTimerText = new Runnable() {
            @Override
            public void run() {
                int hours = seconds / 3600;
                int minutes = (seconds % 3600) / 60;
                int secs = seconds % 60;
                final TextView timerText = (TextView)findViewById(R.id.sleepTimerTV);


                String time = String.format(Locale.getDefault(), "%dh %02dmin %02ds", hours, minutes, secs);
                //Log.i("Timer", time);

                //!! UI elementov se ne da spremijat v run, samo v runOnUiThread
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        timerText.setText(time);
                    }
                });

                seconds++;
            }
        };
        executorTimer.scheduleAtFixedRate(setTimerText, 0, 1, TimeUnit.SECONDS);



    }


    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    private void scanLeDevice() {
        Log.i("scanLeDevice", "started");
        bluetoothAdapter.startLeScan(leScanCallback);
    }


    private final BluetoothAdapter.LeScanCallback leScanCallback = new BluetoothAdapter.LeScanCallback() {
        @Override
        public void onLeScan(final BluetoothDevice device, int rssi, byte[] scanRecord) {
            Log.i("onLeScan", "scanning");
            runOnUiThread(new Runnable() {
                @RequiresApi(api = Build.VERSION_CODES.JELLY_BEAN_MR2)
                @Override
                public void run() {
                    Log.i("onLeScan", device.toString());
                    if (device.getName().equals(DEVICE_NAME)) {
                        connectToDevice(device);
                        bluetoothAdapter.stopLeScan(leScanCallback);

                    }
                }
            });
        }
    };

    @RequiresApi(api = Build.VERSION_CODES.JELLY_BEAN_MR2)
    public void connectToDevice(BluetoothDevice device){
        Log.i("connectToDevice", "Connecting to device...");
        gatt = device.connectGatt(this, false, gattCallback);
    }


    private final BluetoothGattCallback gattCallback = new BluetoothGattCallback() {
        @RequiresApi(api = Build.VERSION_CODES.JELLY_BEAN_MR2)
        @Override
        public void onConnectionStateChange(BluetoothGatt gatt, int status, int newState) {
            Log.i("onConnectionStateChange", "Status: " + status);
            switch (newState) {
                case BluetoothProfile.STATE_CONNECTED:
                    Log.i("gattCallback", "STATE_CONNECTED");

                    gatt.discoverServices();
                    break;
                case BluetoothProfile.STATE_DISCONNECTED:
                    Log.e("gattCallback", "STATE_DISCONNECTED");
                    break;
                default:
                    Log.e("gattCallback", "STATE_OTHER");
            }

        }

        @RequiresApi(api = Build.VERSION_CODES.JELLY_BEAN_MR2)
        @Override
        public void onServicesDiscovered(BluetoothGatt gatt, int status) {
            super.onServicesDiscovered(gatt, status);
            List<BluetoothGattService> services = gatt.getServices();
            for(BluetoothGattService service : services){
                //Log.i("Services", String.valueOf(service.getUuid()));
                if( service.getUuid().equals(UUID.fromString(SERVICE_UUID))) {
                    /* test
                    List<BluetoothGattCharacteristic> chars = service.getCharacteristics();
                    for (BluetoothGattCharacteristic character : chars){
                        Log.i("Chars", String.valueOf(character.getUuid()));
                    }
                     */
                    BluetoothGattCharacteristic characteristic = service.getCharacteristic(UUID.fromString(CHAR_UUID));
                    //Log.i("Service", String.valueOf(service.getUuid()));
                    //Log.i("Characteristic", String.valueOf(characteristic.getUuid()));
                    for (BluetoothGattDescriptor descriptor : characteristic.getDescriptors()) {
                        Log.i("Descriptor", String.valueOf(descriptor));
                        descriptor.setValue( BluetoothGattDescriptor.ENABLE_INDICATION_VALUE);
                        descriptor.setValue( BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE);
                        gatt.writeDescriptor(descriptor);
                    }

                    gatt.setCharacteristicNotification(characteristic, true);
                }
            }

        }

        @RequiresApi(api = Build.VERSION_CODES.JELLY_BEAN_MR2)
        @Override
        public void onCharacteristicRead(BluetoothGatt gatt,
                                         BluetoothGattCharacteristic
                                                 characteristic, int status) {
            //Log.i("onCharacteristicRead", characteristic.toString());
            gatt.disconnect();
        }

        //-----------------------------------------------------------------------------------------------------------------------------------------------
        //prejmi, shrani v buffer, izpisi nov motilec
        @RequiresApi(api = Build.VERSION_CODES.JELLY_BEAN_MR2)
        @Override
        public void onCharacteristicChanged(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic) {
            super.onCharacteristicChanged(gatt, characteristic);
            int newValue = characteristic.getIntValue(FORMAT_SINT16, 0);
            BTrecieved = newValue;
            disruptor = intToStringTable.get(BTrecieved);
            printDisruptorText.setText(disruptor + "\n" + BTrecieved);
            Calendar timeNow = Calendar.getInstance();
            textWriteBuffer[currentPos] = timeNow.getTimeInMillis()+","+disruptor+";";
            currentPos++;

            Log.i("onCharacteristicChanged", String.valueOf(newValue));
        }

    };

    public void fileWrite() throws IOException {
        FileOutputStream stream = new FileOutputStream(file, true);
        try {
            for(int i = 0; i < currentPos-1; i++){
                stream.write(textWriteBuffer[i].getBytes());
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            stream.close();
        }
        currentPos = 0;
    }

    public void goToMainActivity(View view) {
        Intent intent = new Intent(this, MainActivity.class);
        finish();
        startActivity(intent);

    }

    @RequiresApi(api = Build.VERSION_CODES.JELLY_BEAN_MR2)
    @Override
    protected void onDestroy() {
        Calendar timeNow = Calendar.getInstance();
        textWriteBuffer[currentPos] = timeNow.getTimeInMillis()+",idle;";
        currentPos++;
        try {
            fileWrite();
        } catch (IOException e) {
            e.printStackTrace();
        }
        bluetoothAdapter.stopLeScan(leScanCallback);
        if(!(gatt == null)) {
            gatt.close();
            gatt = null;
        }
        //Log.i("onDestroy", "almost done...");
        super.onDestroy();
    }

    @RequiresApi(api = Build.VERSION_CODES.JELLY_BEAN_MR2)
    @Override
    public void onBackPressed() {
        Intent intent = new Intent(this, MainActivity.class);
        finish();
        startActivity(intent);
    }


}